import { globalLookup } from "./global";

export const sidebarSelectedWires = {
    lastLink: undefined,
    pipeFn: [],
    togglePipe() {
        const lastLink = this.lastLink;
        for (const fn of this.pipeFn) {
            fn(lastLink);
        }
    },

    /**
     * 
     * @param {(link: string) => void} callback 
     */
    addToPipe(callback) {
        this.pipeFn.push(callback);
    },

    has(link) {
        return globalLookup.sidebarSelectedWireLinks.has(link);
    },

    add(link) {
        globalLookup.sidebarSelectedWireLinks.add(link);
        // 让渲染进行高亮
        globalLookup.setRenderOption(link, 'highlight', 1);
        globalLookup.render({ type: 'wave-only' });
        this.lastLink = link;
        this.togglePipe();
    },

    /**
     * @description 批量加入 link
     * @param {string[]} links 
     */
    addLinks(links) {
        for (const link of links) {
            globalLookup.sidebarSelectedWireLinks.add(link);
            globalLookup.setRenderOption(link, 'highlight', 1);
            this.lastLink = link;
        }

        globalLookup.render({ type: 'wave-only' });
        this.togglePipe();
    },

    delete(link) {
        globalLookup.sidebarSelectedWireLinks.delete(link);
        if (this.lastLink === link) {
            this.lastLink = undefined;
            for (const link of globalLookup.sidebarSelectedWireLinks) {
                this.lastLink = link;
                break;
            }
        }
        globalLookup.setRenderOption(link, 'highlight', 0);
        globalLookup.render({ type: 'wave-only' });
        this.togglePipe();
    },

    clear() {
        this.lastLink = undefined;
        for (const link of globalLookup.sidebarSelectedWireLinks) {
            globalLookup.setRenderOption(link, 'highlight', 0);
        }

        if (globalLookup.sidebarSelectedWireLinks.size > 0) {
            globalLookup.render({ type: 'wave-only' });
        }

        globalLookup.sidebarSelectedWireLinks.clear();
        this.togglePipe();
    }
};